@@ -41,10 +41,7 @@ typedef struct _GstFlxDec  GstFlxDec;
 struct _GstFlxDec {
   GstElement element;
 
-  GstPad *sinkpad, *srcpad;
-
-  GstSegment segment;
-  gboolean need_segment;
+  GstPad *sinkpad,*srcpad;
 
   gboolean active, new_meta;
 
@@ -82,8 +79,6 @@ struct _GstFlxDecClass {
 /* Standard function returning type information. */
 GType gst_flxdec_get_type(void);
 
-GST_ELEMENT_REGISTER_DECLARE (flxdec);
-
 G_END_DECLS
 
 #endif /* __GST_FLX_DECODER_H__ */
