@@ -163,11 +163,11 @@ handle_buffer_measuring (GstSplitMuxPartReader * reader,
 
   GST_DEBUG_OBJECT (reader, "Pad %" GST_PTR_FORMAT
       " incoming PTS %" GST_TIME_FORMAT
-      " DTS %" GST_TIME_FORMAT " offset by %" GST_TIME_FORMAT
+      " DTS %" GST_TIME_FORMAT " offset by %" GST_STIME_FORMAT
       " to %" GST_TIME_FORMAT, part_pad,
       GST_TIME_ARGS (GST_BUFFER_DTS (buf)),
       GST_TIME_ARGS (GST_BUFFER_PTS (buf)),
-      GST_TIME_ARGS (offset), GST_TIME_ARGS (ts));
+      GST_STIME_ARGS (offset), GST_TIME_ARGS (ts));
 
   if (GST_CLOCK_TIME_IS_VALID (ts)) {
     if (GST_BUFFER_DURATION_IS_VALID (buf))
@@ -737,8 +737,11 @@ new_decoded_pad_added_cb (GstElement * element, GstPad * pad,
   /* Look up or create the output pad */
   if (reader->get_pad_cb)
     out_pad = reader->get_pad_cb (reader, pad, reader->cb_data);
-  if (out_pad == NULL)
+  if (out_pad == NULL) {
+    GST_DEBUG_OBJECT (reader,
+        "No output pad for %" GST_PTR_FORMAT ". Ignoring", pad);
     return;
+  }
 
   /* Create our proxy pad to interact with this new pad */
   proxy_pad = gst_splitmux_part_reader_new_proxy_pad (reader, out_pad);
@@ -911,16 +914,16 @@ type_found (GstElement * typefind, guint probability,
     return;
   }
 
-  gst_element_set_locked_state (demux, TRUE);
-  gst_bin_add (GST_BIN_CAST (reader), demux);
-  gst_element_link_pads (reader->typefind, "src", demux, NULL);
-  gst_element_sync_state_with_parent (reader->demux);
-  gst_element_set_locked_state (demux, FALSE);
-
   /* Connect to demux signals */
   g_signal_connect (demux,
       "pad-added", G_CALLBACK (new_decoded_pad_added_cb), reader);
   g_signal_connect (demux, "no-more-pads", G_CALLBACK (no_more_pads), reader);
+
+  gst_element_set_locked_state (demux, TRUE);
+  gst_bin_add (GST_BIN_CAST (reader), demux);
+  gst_element_link_pads (reader->typefind, "src", demux, NULL);
+  gst_element_set_state (reader->demux, GST_STATE_TARGET (reader));
+  gst_element_set_locked_state (demux, FALSE);
 }
 
 static void
@@ -991,7 +994,6 @@ gst_splitmux_part_reader_src_query (GstSplitMuxPartReader * part,
     return FALSE;
 
   ret = gst_pad_peer_query (target, query);
-  gst_object_unref (GST_OBJECT_CAST (target));
 
   if (ret == FALSE)
     goto out;
@@ -1182,6 +1184,18 @@ gst_splitmux_part_reader_activate (GstSplitMuxPartReader * reader,
   return TRUE;
 }
 
+gboolean
+gst_splitmux_part_reader_is_active (GstSplitMuxPartReader * part)
+{
+  gboolean ret;
+
+  SPLITMUX_PART_LOCK (part);
+  ret = part->active;
+  SPLITMUX_PART_UNLOCK (part);
+
+  return ret;
+}
+
 void
 gst_splitmux_part_reader_deactivate (GstSplitMuxPartReader * reader)
 {
